package com.hanback.mdscpe;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

public class MdscpeActivity extends Activity {
    /** Called when the activity is first created. */
    static { System.loadLibrary("mdscpe");}
    private native int MdscpeControl();
    private native int MdscpeCal();

    static final int 	RECV_DATA=100;
	static final int	DIALOG_CAL_MSG=101;
	static final int	CAL_ID=102;
	
	static final int	ERR_FD=-1; 
	static final int 	ERR_NO_CAL=-2;
	static final int 	ERR_CAL_FAIL=-3;
	
	
	public int	value;
	boolean		f_read;
	boolean		f_cal;
	public int	cal_start;
	
	EventHandler	m_eventHandler;
	MdscpeThread	m_mdscpeThread;
	SampleView 		mView; 
	TextView 		mdscpe_text;
	LinearLayout	mdscpe_layout;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mdscpe_text = (TextView)findViewById(R.id.mdscpe_value);
        
        m_eventHandler = new EventHandler();
        m_mdscpeThread = new MdscpeThread();
        mView = new SampleView(this);
        
        mdscpe_layout=(LinearLayout)findViewById(R.id.mdscpe_layout);
        mdscpe_layout.addView(mView);
        
        value=0;
        cal_start=0;
        f_read=true;
        f_cal=false;
        m_mdscpeThread.start();
    }
    
    @Override
    public void onDestroy() {
    	Log.d("Mdscpe", "onDestroy() Mdscpe");
    	super.onDestroy();
    }
    
    public void onStop() {
    	Log.d("Mdscpe", "onStop() Mdscpe");
    	f_read=false; f_cal=false;
    	finish();
    	super.onStop();
    }
       

    @Override
    public void onRestart() {
    	Log.d("Mdscpe", "onRestart() Mdscpe");
    	f_read=true; f_cal=false;
    	super.onRestart();
    }
    
    @Override
    public void onResume() {
        super.onResume();
    	Log.d("Mdscpe", "onResume() Mdscpe");
    }

    @Override
    public void onPause() {
    	Log.d("Mdscpe", "onPause() Mdscpe");
        super.onPause();
    }
    
    public boolean onCreateOptionsMenu(Menu menu){
    	menu.add(Menu.NONE, CAL_ID, Menu.NONE, "Calibration");
    	return (super.onCreateOptionsMenu(menu));
    }
    
    public boolean onOptionsItemSelected(MenuItem item) {
    	return (applyMenuChoice(item)||
    			super.onContextItemSelected(item));
    }
    
    private boolean applyMenuChoice(MenuItem item) {
    	Log.d("Mdscpe", "onPause() Mdscpe");
    	switch (item.getItemId()){
    	case CAL_ID:
    		calCompass();
    		return (true);
    	}
    	return (false);
    }
    
    protected Dialog onCreateDialog(int id)
    {
    	Dialog dialog;
    	dialog = null;
    	
    	switch(id)
    	{
    	case DIALOG_CAL_MSG:
    		ProgressDialog progressDlg;
    		progressDlg = new ProgressDialog(MdscpeActivity.this);
    		progressDlg.setMessage("Calibration մϴ.. õõ 360 ");
    		progressDlg.setCancelable(false);
    		progressDlg.show();
    		dialog = (Dialog)progressDlg;
    		break;
    		
    	default:
    		dialog=null;
    		break;
    	}
    	
    	return dialog;
    }
    
    public int calCompass() {
    	f_read=false;
    	showDialog(DIALOG_CAL_MSG);
    	f_cal=true;
    	
		Handler mHandle2 = new Handler();
        mHandle2.postDelayed(new Runnable(){
         	public void run()
         	{
         		removeDialog(DIALOG_CAL_MSG);
         		f_read=true;
         	}
         }, 8000);
    	return 0;
    }
    
    public class EventHandler extends Handler {
    	EventHandler() {}
    	public void handleMessage(Message msg) 
      	{
    		
      		try
      		{
      			if(msg.what == RECV_DATA)
    	    	{
    				//Log.d("CDS","Value : " + cds_value);
    				if(value>0 && value<=360) {
    					mdscpe_text.setText("Value : " + value);
    					mView.invalidate();
    				}
    	    	}
    			else if(msg.what == ERR_FD) 
    			{
    				f_read=false;
    				new AlertDialog.Builder(mdscpe_layout.getContext())
    				.setTitle("<ERROR>")
    				.setMessage("MDSCPE : ̽ ̹ Ȯϼ!!")
    				.setNeutralButton("ݱ", new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							finish();
						}
					})
					.show();
    			}
    			else if(msg.what == ERR_NO_CAL) 
    			{
    				f_read = false;
    				new AlertDialog.Builder(mdscpe_layout.getContext())
    				.setTitle("<Warning>")
    				.setMessage("MDSCPE : CAL ư  Calibaration մϴ!!")
    				.setNeutralButton("CAL", new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							Log.d("Mdscpe","calCompass()");
	    					calCompass();
						}
					})
					.setNegativeButton("", new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							finish();
						}
					})
					.show();
    			}
    			else if(msg.what == ERR_CAL_FAIL) 
    			{
    				f_read=false;
    				new AlertDialog.Builder(mdscpe_layout.getContext())
    				.setTitle("ERROR")
    				.setMessage("MDSCPE : Calibration Ͽϴ!!")
    				.setNeutralButton("ݱ", new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							finish();
						}
					})
					.show();
    			}
      		}
      		catch(Exception e) {}
    	    		
      		return;
      	}
	}
    
    public class MdscpeThread extends Thread{
    	public void run() {
    		
    		try {
    			while(true){
    				if(f_read) {
    					value = MdscpeControl();
    					//Log.d("Mdscpe","Value : " + value);
    					Message msg1 = m_eventHandler.obtainMessage();
    					if(value>0)
    						msg1.what= RECV_DATA;
    					else
    						msg1.what= value;
    					m_eventHandler.sendMessage(msg1);
    				}
    				if(f_cal) {
    					MdscpeCal();
    					f_cal=false;
    				}
    				Thread.sleep(100);
    			}
    		}
    		catch (Exception e) {
    			
    		}
    	}
    }
    
    private class SampleView extends View { 
        private Paint   mPaint = new Paint(); 
        private Path    mPath = new Path(); 
        private boolean mAnimate;
        
        public SampleView(Context context) { 
            super(context); 
 
            // Construct a wedge-shaped path 
            mPath.moveTo(0, -50); 
            mPath.lineTo(-20, 60); 
            mPath.lineTo(0, 50); 
            mPath.lineTo(20, 60); 
            mPath.close(); 
        } 
 
        @Override protected void onDraw(Canvas canvas) { 
            Paint paint = mPaint; 
 
            canvas.drawColor(Color.WHITE); 
 
            paint.setAntiAlias(true); 
            paint.setColor(Color.RED); 
            paint.setStyle(Paint.Style.FILL); 
 
            int w = canvas.getWidth(); 
            int h = canvas.getHeight(); 
            int cx = w / 2; 
            int cy = h / 2; 
 
            canvas.translate(cx, cy);
            if(value<=360)
            	canvas.rotate(360-value+180);
            canvas.drawPath(mPath, mPaint); 
        } 
 
        @Override 
        protected void onAttachedToWindow() { 
            mAnimate = true; 
            super.onAttachedToWindow(); 
        } 
 
        @Override 
        protected void onDetachedFromWindow() { 
            mAnimate = false; 
            super.onDetachedFromWindow(); 
        } 
    }    
}