/*
 * Base Board 7-Segment Device Driver
 *  Hanback Electronics Co.,ltd
 * File : baseseg.c
 * Date : October,2010
 */ 

//   
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <asm/fcntl.h>
#include <linux/ioport.h>
#include <linux/delay.h>

#include <asm/ioctl.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <mach/regs-gpio.h>
#include <mach/gpio-bank.h>


#define DRIVER_AUTHOR		"hanback"		//  
#define DRIVER_DESC		"Base 7-Segment program"	// ⿡  

#define	SEGMENT_MAJOR		249			// ̽ ֹȣ
#define	SEGMENT_NAME		"BASESEG"		// ̽ ̸
#define SEGMENT_MODULE_VERSION	"BASESEG PORT V0.1"	// ̽ 

#define GPJ2CON         (*(unsigned long *)S5PV210_GPJ2CON)
#define GPJ2DAT         (*(unsigned long *)S5PV210_GPJ2DAT)
#define GPJ2PUD         (*(unsigned long *)S5PV210_GPJ2PUD)
#define GPJ3CON         (*(unsigned long *)S5PV210_GPJ3CON)
#define GPJ3DAT         (*(unsigned long *)S5PV210_GPJ3DAT)
#define GPJ3PUD         (*(unsigned long *)S5PV210_GPJ3PUD)



//Global variable
static unsigned int segment_usage = 0;

// define functions...
int baseseg_open(struct inode *inode, struct file *filp)
{
	if(segment_usage != 0) return -EBUSY;
	
        GPJ2CON &= (~0xFFFFFFFF);       GPJ2CON |= 0x11111111;
        GPJ2PUD &= (~0x33333333);       GPJ2PUD |= 0x55555555; // PULL Down
        GPJ2DAT = 0x00000000;

        GPJ3CON &= (~0x000000FF);       GPJ3CON |= 0x00000011;
        GPJ3PUD &= (~0x0000000F);       GPJ3PUD |= 0x0000000A; // PULL UP
        GPJ3DAT &= (~0x00000003);       GPJ3DAT |= 0x00000003; // off

	GPJ2DAT = 0x3f;
        GPJ3DAT &= (~0x00000003);      

	segment_usage = 1;	
	return 0; 
}

int baseseg_release (struct inode *inode, struct file *filp)
{

        GPJ3DAT |= 0x00000003; //off      
	segment_usage = 0;
	return 0;
}

unsigned short Getsegmentcode (short x)
{
	unsigned short code;
	switch (x) {
		case 0x0 : code = 0x3f; break;
		case 0x1 : code = 0x06; break;
		case 0x2 : code = 0x5B; break;
		case 0x3 : code = 0x4F; break;
		case 0x4 : code = 0x66; break;
		case 0x5 : code = 0x6D; break;
		case 0x6 : code = 0x7D; break;
		case 0x7 : code = 0x27; break;
		case 0x8 : code = 0x7F; break;
		case 0x9 : code = 0x6F; break;
		case 0xa : code = 0x77; break;
		case 0xb : code = 0x7C; break;
		case 0xc : code = 0x58; break;
		case 0xd : code = 0x5E; break;	
		case 0xe : code = 0x79; break;
		case 0xf : code = 0x71; break;				
		default : code = 0; break;
	}
	return code;
}						

ssize_t baseseg_write(struct file *inode, const char *gdata, size_t length, loff_t *off_what)
{
    unsigned char data[2];
    unsigned int num,ret;

    //  ޸ gdata Ŀ ޸ num nŭ  
    ret=copy_from_user(&num,gdata,4);	

	data[1]=Getsegmentcode(num/10);
	data[0]=Getsegmentcode(num%10);

	// print
        GPJ3DAT |= 0x00000003; //off      
        GPJ3DAT &= (~0x00000002);       
        GPJ2DAT = (unsigned int)data[0];
        mdelay(5);	
        GPJ3DAT |= 0x00000002;       
        GPJ3DAT &= (~0x00000001);       
        GPJ2DAT = (unsigned int)data[1];
        mdelay(1);	

    return length;
}

static int baseseg_ioctl(struct inode *inode, struct file *flip, unsigned int cmd, unsigned long arg)
{
    return 0;
}
							    

struct file_operations baseseg_fops = 
{
	.owner		= THIS_MODULE,
	.open		= baseseg_open,
	.write		= baseseg_write,
	.release	= baseseg_release,
	.ioctl		= baseseg_ioctl,
};

int baseseg_init(void)
{
	int result;

	result = register_chrdev(SEGMENT_MAJOR, SEGMENT_NAME, &baseseg_fops);
	if (result < 0) {
		printk(KERN_WARNING"Can't get any major\n");
		return result;
	}

	// major ȣ Ѵ.
	printk(KERN_INFO"Init Module, Base 7-Segment Major Number : %d\n", SEGMENT_MAJOR);
	return 0;
}

//  Ŀο 
void baseseg_exit(void)
{
	//  ̽ ̹ Ѵ.
	unregister_chrdev(SEGMENT_MAJOR,SEGMENT_NAME);

	printk("driver: %s DRIVER EXIT\n", SEGMENT_NAME);
}

module_init(baseseg_init);	//    ȣǴ Լ
module_exit(baseseg_exit);	//    ȣǴ Լ

MODULE_AUTHOR(DRIVER_AUTHOR);	//  
MODULE_DESCRIPTION(DRIVER_DESC);// ⿡  
MODULE_LICENSE("Dual BSD/GPL");	//  ̼ 

