/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <string.h>
#include <jni.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/mman.h>
#include <errno.h>
#include <android/log.h>

struct mdscpe_sensor_command
{
	short xoff;
	short yoff;
	short xsc;
	short ysc;
	short k;
};

static struct mdscpe_sensor_command envir;
static int fd = -1;

enum ERROR_LIST {
	ERR_FD=-1, ERR_NO_CAL=-2, ERR_CAL_FAIL=-3
};

#define SENSOR_MAGIC		'M'
#define SENSOR_CALIB_START	_IO(SENSOR_MAGIC,0)
#define SENSOR_CALIB_WRITE	_IO(SENSOR_MAGIC,1)
#define SENSOR_CALIB_READ	_IOR(SENSOR_MAGIC,2,struct mdscpe_sensor_command)
#define SENSOR_DATA_READ	_IOR(SENSOR_MAGIC,3,short)
#define DEVICE_NAME		"/dev/mdscpe_sensor"
#define CAL_TXT_NAME		"/etc/compass_cal_data.txt"

jint
Java_com_hanback_mdscpe_MdscpeActivity_MdscpeCal( JNIEnv* env,
                                                  jobject thiz)
{
	FILE *file;
	int fd;

	if((fd = open(DEVICE_NAME,O_RDWR))<0)
		return ERR_FD;
	
	if(file=fopen(CAL_TXT_NAME, "w")) {
		// MDS-CPE Calibration Start..
		ioctl(fd, SENSOR_CALIB_START);
		usleep(1000*200);

		// Read Calibration data
		ioctl(fd, SENSOR_CALIB_READ, &envir, sizeof(struct mdscpe_sensor_command));

		// Calibration Data to compass_cal_data.txt
		fprintf(file, "%x %x %x %x %x", envir.xoff, envir.yoff, envir.xsc, envir.ysc, envir.k);
	}
	else 	// Cal error
		return ERR_CAL_FAIL;

	fclose(file);
	close(fd);

	return 0;
}

jint
Java_com_hanback_mdscpe_MdscpeActivity_MdscpeControl( JNIEnv* env,
                                                  jobject thiz)
{
	FILE *file;
	int fd=0;
	int value=0;
	short buf[5];

	if((fd = open(DEVICE_NAME,O_RDWR))<0)
		return ERR_FD;

	if (file=fopen(CAL_TXT_NAME, "r")) {
		fscanf( file, "%x %x %x %x %x", &buf[0], &buf[1], &buf[2], &buf[3], &buf[4]);

		// Write Calib Data...
		write(fd,buf,sizeof(buf));
		ioctl(fd, SENSOR_CALIB_WRITE);
	}
	else // Calibration need
		return ERR_NO_CAL;

	fclose(file);

	ioctl(fd, SENSOR_DATA_READ, &value,sizeof(value));
	usleep(1000 * 100);
	close(fd);

	return value;
}

